//Author: Ian Goodfellow
//File: svlHarrisCornerDetector.h
//Description: Implements svlInterestPointDetector using Harris corner detection

#pragma once

#include "svlInterestPointDetector.h"

class svlHarrisCornerDetector : public svlInterestPointDetector
{
 public:
  svlHarrisCornerDetector(int blockSize, int apertureSize, double k, int _maxWinWidth, int maxWinHeight, double responseThreshold);
  svlHarrisCornerDetector(XMLNode & node);
  virtual ~svlHarrisCornerDetector() { }
  void findInterestPoints(const IplImage * img, vector<CvPoint> & output);
  void setResponseShowing(bool show ) { _showResponse = show; }

  void write(ofstream & ofs) const;

  bool operator!=(const svlInterestPointDetector & other) const;

 protected:
  
  //Arguments to cvCornerHarris
  int _blockSize;
  int _apertureSize;
  double _k;

  //Harris response is filtered to find local maxima within this window size
  int _maxWinWidth;
  int _maxWinHeight;

  //Only interest points with at least this Harris threshold are returned
  double _responseThreshold; 

  bool _showResponse;
  
};
