/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    svlMapping.cpp
** AUTHOR(S):   Paul Baumstarck <pbaumstarck@stanford.edu>
** DESCRIPTION:
**  Class implementing forward and backward permutation.
**
*****************************************************************************/

#include "svlMapping.h"

// ****** Con/Destructors ******

svlMapping::svlMapping() {
	// do nothing
}

svlMapping::~svlMapping() {
	// do nothing
}

// ****** General functions ******

void svlMapping::clear() {
	if ( !AtoB.empty() ) AtoB.clear();
	if ( !BtoA.empty() ) BtoA.clear();
}

void svlMapping::addAtoB(int a, int b) {
	AtoB[a] = MappingIndex(b);
}

void svlMapping::addBtoA(int b, int a) {
	BtoA[b] = MappingIndex(a);
}

void svlMapping::addAtofromB(int a, int b) {
	AtoB[a] = MappingIndex(b);
	BtoA[b] = MappingIndex(a);
}

int svlMapping::getAtoB(int a) {
	return AtoB[a].i;
}

int svlMapping::getBtoA(int b) {
	return BtoA[b].i;
}

// ****** Iterators ******

void svlMapping::setIterABegin() {
	itA = AtoB.begin();
}

void svlMapping::setIterBBegin() {
	itB = BtoA.begin();
}

void svlMapping::incIterA() {
	++itA;	
}

void svlMapping::incIterB() {
	++itB;
}

pair<int,int> svlMapping::getIterA() {
	return make_pair(itA->first,itA->second.i);
}

pair<int,int> svlMapping::getIterB() {
	return make_pair(itB->first,itB->second.i);
}

bool svlMapping::iterAEnd() {
	return itA == AtoB.end();
}

bool svlMapping::iterBEnd() {
	return itB == BtoA.end();
}

// ****** Weird functions ******

vector<int> svlMapping::getRingFromA(int a) {
	vector<int> ret;
	int i = a;
	ret.push_back(i); // A
	do {
		ret.push_back( i = AtoB[i].i ); // B
		if ( i == -1 ) break;
		ret.push_back( i = BtoA[i].i ); // A
		if ( i == -1 ) break;
	} while ( i != a );
	return ret;
}
